@@ -470,6 +470,7 @@ impl Enum {
 pub struct Field {
     message: Weak<Message>,
     rust_ident: String,
+    rust_ident_unesc: String,
     rust_oneof_ident: String,
     lazy_proto_type: OnceCell<FieldType>,
     proto_name: String,
@@ -492,7 +493,7 @@ impl Field {
             })?
             .to_string();
         let proto_type_name = proto.type_name.clone().unwrap_or_default().to_string();
-        let proto_type_enum = proto.type_.clone().ok_or(ErrorKind::InternalError {
+        let proto_type_enum = proto.r#type.clone().ok_or(ErrorKind::InternalError {
             detail: "currently we are assuming the field type enum is always set.".to_string(),
         })?;
         let proto_label = proto.label.clone().ok_or(ErrorKind::InternalError {
@@ -504,6 +505,7 @@ impl Field {
         Ok(Self {
             message: Clone::clone(&message),
             rust_ident: get_keyword_safe_ident(&to_lower_snake_case(&proto_name)).to_string(),
+            rust_ident_unesc: to_lower_snake_case(&proto_name).to_string(),
             rust_oneof_ident: get_keyword_safe_ident(&to_camel_case(&proto_name)).to_string(),
             proto_name,
             proto_type_name,
@@ -520,6 +522,9 @@ impl Field {
     pub fn rust_ident(&self) -> &str {
         &self.rust_ident
     }
+    pub fn rust_ident_unesc(&self) -> &str {
+        &self.rust_ident_unesc
+    }
     pub fn proto_name(&self) -> &str {
         &self.proto_name
     }
@@ -624,14 +629,8 @@ impl Field {
             FieldType::Bool => "bool".to_string(),
             FieldType::Group => Err(ErrorKind::GroupNotSupported)?,
             FieldType::Enum2(e) | FieldType::Enum3(e) => upgrade(&e)?.rust_path(),
-            FieldType::String => format!(
-                "Self::Field{number}StringType<'this>",
-                number = self.number()
-            ),
-            FieldType::Bytes => format!(
-                "Self::Field{number}BytesType<'this>",
-                number = self.number()
-            ),
+            FieldType::String => "&'this str".to_string(),
+            FieldType::Bytes => "&'this [u8]".to_string(),
             FieldType::Message(_) => format!(
                 "Self::Field{number}MessageType<'this>",
                 number = self.number()
@@ -655,20 +654,8 @@ impl Field {
             FieldType::Bool => "bool".to_string(),
             FieldType::Group => Err(ErrorKind::GroupNotSupported)?,
             FieldType::Enum2(e) | FieldType::Enum3(e) => upgrade(&e)?.rust_path(),
-            FieldType::String => format!(
-                "<{trait_impl} as {trait_path}>::Field{number}StringType<{lt}>",
-                lt = lt,
-                trait_impl = trait_impl,
-                trait_path = self.message()?.rust_trait_path(),
-                number = self.number(),
-            ),
-            FieldType::Bytes => format!(
-                "<{trait_impl} as {trait_path}>::Field{number}BytesType<{lt}>",
-                lt = lt,
-                trait_impl = trait_impl,
-                trait_path = self.message()?.rust_trait_path(),
-                number = self.number(),
-            ),
+            FieldType::String => format!("&{lt} str", lt = lt),
+            FieldType::Bytes => format!("&{lt} [u8]", lt = lt),
             FieldType::Message(_) => format!(
                 "<{trait_impl} as {trait_path}>::Field{number}MessageType<{lt}>",
                 lt = lt,
@@ -715,6 +702,46 @@ impl Field {
         Ok(matches!(self.field_label(), Ok(FieldLabel::Repeated)))
     }
 
+    pub fn oneof_field_type(&self) -> Result<String> {
+        Ok(match self.field_type()? {
+            FieldType::Double => "f64".to_string(),
+            FieldType::Float => "f32".to_string(),
+            FieldType::Int32 => "i32".to_string(),
+            FieldType::Int64 => "i64".to_string(),
+            FieldType::UInt32 => "u32".to_string(),
+            FieldType::UInt64 => "u64".to_string(),
+            FieldType::SInt32 => "i32".to_string(),
+            FieldType::SInt64 => "i64".to_string(),
+            FieldType::Fixed32 => "u32".to_string(),
+            FieldType::Fixed64 => "u64".to_string(),
+            FieldType::SFixed32 => "i32".to_string(),
+            FieldType::SFixed64 => "i64".to_string(),
+            FieldType::Bool => "bool".to_string(),
+            FieldType::Group => Err(ErrorKind::GroupNotSupported)?,
+            FieldType::Enum2(e) | FieldType::Enum3(e) => upgrade(&e)?.rust_path(),
+            FieldType::String => "<IsOwned as ::puroro::internal::bool::BoolType>\
+                ::Choose<::std::string::String, &'msg str>"
+                .to_string(),
+            FieldType::Bytes => "<IsOwned as ::puroro::internal::bool::BoolType>\
+                ::Choose<::std::vec::Vec<u8>, &'msg [u8]>"
+                .to_string(),
+            FieldType::Message(m) => {
+                let simple_message_type = upgrade(&m)?.rust_impl_path("Simple");
+                let trait_getter_type = format!(
+                    "<T as {trait_path}>::Field{number}MessageType<'msg>",
+                    trait_path = self.message()?.rust_trait_path(),
+                    number = self.number(),
+                );
+                format!(
+                    "<IsOwned as ::puroro::internal::bool::BoolType>\
+                    ::Choose<::std::boxed::Box<{message_type}>, {trait_getter_type}>",
+                    message_type = simple_message_type,
+                    trait_getter_type = trait_getter_type,
+                )
+            }
+        })
+    }
+
     pub fn simple_field_type(&self) -> Result<String> {
         let scalar_type = self.simple_scalar_field_type()?;
         Ok(match self.field_label()? {
@@ -760,18 +787,8 @@ impl Field {
     pub fn single_field_type(&self) -> Result<String> {
         let scalar_type = self.single_scalar_field_type()?;
         Ok(match self.field_label()? {
-            FieldLabel::OneofField => scalar_type,
-            FieldLabel::Required | FieldLabel::Optional => {
-                format!("::std::option::Option<{}>", scalar_type)
-            }
-            FieldLabel::Unlabeled => {
-                if matches!(self.field_type(), Ok(FieldType::Message(_))) {
-                    format!("::std::option::Option<{}>", scalar_type)
-                } else {
-                    scalar_type
-                }
-            }
             FieldLabel::Repeated => "RepeatedType".to_string(),
+            _ => scalar_type,
         })
     }
 
